/***************************************************************************
                          cdownloadmanager.cpp  -  description
                             -------------------
    begin                : Don Mai 16 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>

#include <dclib/dcobject.h>
#include <dclib/cconfig.h>
#include <dclib/cdir.h>
#include <dclib/cservermanager.h>

#ifndef WIN32
#include <unistd.h>
#endif

#include "cdownloadmanager.h"

extern CConfig * pDCLibConfig;
extern CServerManager * pDCLibServerManager;

CDownloadManager * pDownloadManager = 0;

/** */
CDownloadManager::CDownloadManager()
{
	lTransferID  = time(0);
	lUploadCount = 0;
	iShutdownState = 0;

	pDownloadManager = this;

	pTransferItemList  = new CList<DCTransferItem>;
	pTransferList      = new CList<CTransfer>;
	pExtraUserSlotList = new CList<CExtraUserSlot>;

	Start();
}

/** */
CDownloadManager::~CDownloadManager()
{
	StopListen();

	Stop();

	if(pTransferItemList)
	{
		pTransferItemList->Clear();
		delete pTransferItemList;
		pTransferItemList = 0;
	}

	if (pExtraUserSlotList)
	{
		delete pExtraUserSlotList;
		pExtraUserSlotList = 0;
	}

	if(pTransferList)
	{
		pTransferList->Clear();
		delete pTransferList;
		pTransferList = 0;
	}
}

/** */
void CDownloadManager::SendFileInfo( DCTransferItem * TransferItem, DCTransferFileItem * TransferFileItem, bool bRemoveFile )
{
	CMessageDMFileObject * fo = new CMessageDMFileObject();

	fo->sNick       = TransferItem->sNick;
	fo->sHubName    = TransferItem->sHubName;
	fo->sHubHost    = TransferItem->sHubHost;
	fo->eWaitState  = TransferItem->eState;
	fo->timeTimeout = TransferItem->timeTimeout;

	if ( TransferFileItem )
	{
		fo->sRemoteName      = TransferFileItem->sRemoteName;
		fo->sLocalName       = TransferFileItem->sLocalName;
		fo->sLocalPath       = TransferFileItem->sLocalPath;
		fo->lSize            = TransferFileItem->lSize;
		fo->lStartPosition   = TransferFileItem->lStartPosition;
		fo->lEndPosition     = TransferFileItem->lEndPosition;
		fo->lCurrentPosition = TransferFileItem->lCurrentPosition;
		fo->eFileState       = TransferFileItem->eState;
		fo->bRemoveFile      = bRemoveFile;
	}

	fo->eType = DC_MESSAGE_FILE_OBJECT;

	if ( DC_DownloadManagerCallBack(fo) == -1 )
		delete fo;
}

/** */
void CDownloadManager::SendTransferInfo( CTransfer * Transfer, bool remove )
{
	CMessageDMTransferObject * to = new CMessageDMTransferObject();

	to->sNick            = Transfer->GetDstNick();
	to->sHost            = Transfer->GetHost();
	to->eState           = Transfer->GetMode();
	to->sRemoteName      = Transfer->GetDstFilename();
	to->lSize            = Transfer->GetLength();
	to->lStartPosition   = Transfer->GetStartPosition();
	to->lCurrentPosition = Transfer->GetCurrentPosition();
	to->lEndPosition     = Transfer->GetEndPosition();
	to->lRate            = Transfer->GetTransferrate();
	to->bRemoveTransfer  = remove;

	to->eType = DC_MESSAGE_TRANSFER_OBJECT;

	if ( DC_DownloadManagerCallBack(to) == -1 )
		delete to;
}

/** */
void CDownloadManager::SendSlotInfo( CExtraUserSlot * Object )
{
	CMessageDMSlotObject * so = new CMessageDMSlotObject();

	so->sNick      = Object->sNick;
	so->sHubName   = Object->sHubName;
	so->iSlots     = Object->iSlots;
	so->bPermanent = Object->bPermanent;

	so->eType = DC_MESSAGE_SLOT_OBJECT;

	if ( DC_DownloadManagerCallBack(so) == -1 )
		delete so;
}

/** */
void CDownloadManager::SendLogInfo( CString message, CTransfer * Transfer )
{
	LogThread.Lock();

	CString s = "";

	CMessageLog * log = new CMessageLog();

	if ( Transfer != 0 )
	{
		s = "[" + Transfer->GetDstNick() + "] ";
	}
	
	s += message;
	
	log->sMessage = s;
	log->eType    = DC_MESSAGE_LOG;

	if ( DC_DownloadManagerCallBack(log) == -1 )
		delete log;

	LogThread.UnLock();
}

/** */
int CDownloadManager::LoadQueue()
{
	int err;

	Lock();

	pTransferItemList->Clear();

	err = pDCLibConfig->LoadDCTra( pTransferItemList );

	// send all files over the callback function
	if ( err == 0 )
	{
		DCTransferItem * TransferItem = 0;

		while( (TransferItem=pTransferItemList->Next(TransferItem)) != 0 )
		{
			DCTransferFileItem * TransferFileItem = 0;

			while( (TransferFileItem=TransferItem->pFileList->Next(TransferFileItem)) != 0 )
			{
				SendFileInfo( TransferItem, TransferFileItem );
			}
		}
	}

	SendLogInfo("Load Queue ...\n");

	UnLock();

	return err;
}

/** */
int CDownloadManager::SaveQueue()
{
	int err;

	Lock();

	err = pDCLibConfig->SaveDCTra( pTransferItemList );

	UnLock();

	return err;
}

/** */
bool CDownloadManager::InitListen( CString & s )
{
	bool res = FALSE;

	if ( Listen.StartListen( pDCLibConfig->GetTCPListenPort() ) == 0 )
	{
		Listen.SetCallBackFunction(DMListenCallBack);

		res = TRUE;
	}
	else
	{
		s = Listen.GetSocketError();
	}

	return res;
}

/** */
void CDownloadManager::StopListen()
{
	Listen.SetCallBackFunction(0);
	Listen.StopListen();
}

/** */
void CDownloadManager::AddUserSlot( CString nick, CString hubname, int slot, bool permanent )
{
	CExtraUserSlot * ExtraUserSlot;
	Lock();

	ExtraUserSlot = 0;

	while( (ExtraUserSlot=pExtraUserSlotList->Next(ExtraUserSlot)) != 0 )
	{
		if ( (ExtraUserSlot->sNick == nick) && (ExtraUserSlot->sHubName == hubname) )
		{
			if ( (slot == 0) && (!permanent) )
			{
				ExtraUserSlot->iSlots     = slot;
				ExtraUserSlot->bPermanent = FALSE;
			}
			else
			{
				if (ExtraUserSlot->bPermanent)
					ExtraUserSlot->iSlots = 0;
				ExtraUserSlot->iSlots     += slot;
				ExtraUserSlot->bPermanent  = permanent;
			}
			break;
		}
	}

	if ( ExtraUserSlot == 0 )
	{
		ExtraUserSlot = new CExtraUserSlot();
		ExtraUserSlot->sNick      = nick;
		ExtraUserSlot->sHubName   = hubname;
		ExtraUserSlot->iSlots     = slot;
		ExtraUserSlot->bPermanent = permanent;

		pExtraUserSlotList->Add(ExtraUserSlot);
	}

	SendSlotInfo(ExtraUserSlot);

	if ( (ExtraUserSlot->iSlots == 0) && (!ExtraUserSlot->bPermanent) )
	{
		pExtraUserSlotList->Del(ExtraUserSlot);
	}

	UnLock();
}

/** */
bool CDownloadManager::CheckUserSlot( CString nick, CString hubname )
{
	bool res = FALSE;
	CExtraUserSlot * ExtraUserSlot = 0;

	Lock();

	while( (ExtraUserSlot=pExtraUserSlotList->Next(ExtraUserSlot)) != 0 )
	{
		if ( ExtraUserSlot->sNick == nick )
		{
			if ( ExtraUserSlot->iSlots > 0 )
			{
				ExtraUserSlot->iSlots--;

				SendSlotInfo(ExtraUserSlot);

				if ( ExtraUserSlot->iSlots == 0 )
				{
					pExtraUserSlotList->Del(ExtraUserSlot);
				}

				res = TRUE;
			}
			else if ( ExtraUserSlot->bPermanent )
			{
				res = TRUE;
			}

			break;
		}
	}

	UnLock();

	return res;
}

/** disconnect all transfers for a clean shutdown */
void CDownloadManager::Shutdown()
{
	CTransfer * Transfer;
	DCTransferItem * TransferItem;

	Listen.StopListen();
	Listen.SetCallBackFunction(0);

	iShutdownState = 1;

	Transfer = 0;

	SendLogInfo("Shutdown download manager ...\n");

	Lock();

	printf("COUNT INCOMING: %ld\n",pTransferList->Count());//fflush(stdout);
	while( (Transfer=pTransferList->Next(Transfer)) != 0 )
	{
		printf("%d\n",Transfer->GetDone());
		Transfer->Disconnect(TRUE);
		
	}

	TransferItem = 0;

	printf("COUNT OTHER: %ld\n",pTransferList->Count());//fflush(stdout);
	while( (TransferItem=pTransferItemList->Next(TransferItem)) != 0 )
	{
		Transfer = 0;

		while ( (Transfer=TransferItem->pTransferList->Next(Transfer)) != 0 )
		{
			printf("COUNT: %ld\n",TransferItem->pTransferList->Count());//fflush(stdout);
			printf("%d\n",Transfer->GetDone());
			Transfer->Disconnect(TRUE);
		}
	}

	UnLock();
}

/** */
bool CDownloadManager::CloseTransfer( CString nick, CString host, CString file )
{
	CTransfer * Transfer;
	DCTransferItem * TransferItem;
	bool found=FALSE;

	Lock();

	Transfer = 0;

	while( (Transfer=pTransferList->Next(Transfer)) != 0 )
	{
		if ( (Transfer->GetDstNick() == nick) &&	
		     (Transfer->GetHost() == host) &&
		     (Transfer->GetDstFilename() == file) )
		{
			Transfer->Disconnect(TRUE);
			found = TRUE;
			break;
		}
	}

	if ( found == FALSE )
	{
		TransferItem = 0;

		while( (TransferItem=pTransferItemList->Next(TransferItem)) != 0 )
		{
			Transfer = 0;

			while ( (Transfer=TransferItem->pTransferList->Next(Transfer)) != 0 )
			{
				if ( (Transfer->GetDstNick() == nick) &&	
					(Transfer->GetHost() == host) &&
					(Transfer->GetDstFilename() == file) )
				{
					Transfer->Disconnect(TRUE);
					found = TRUE;
					break;
				}
			}
		}
	}

	UnLock();

	return found;
}

/** */
bool CDownloadManager::GetUserFileInfo( CString nick, CString hubname, CString hubhost, CString file, CUserFileInfo * UserFileInfo )
{
	bool res = FALSE;
	DCTransferItem * TransferItem;
	DCTransferFileItem * TransferFileItem;

	if (!UserFileInfo)
	{
		return FALSE;
	}

	Lock();

	TransferItem = GetTransferItem(nick,hubname,hubhost);

	if ( TransferItem != 0 )
	{
		UserFileInfo->eWaitState    = TransferItem->eState;
		UserFileInfo->sUserFileList = TransferItem->sUserFileList;

		if ( file != "" )
		{
			TransferFileItem = GetTransferFileItem( TransferItem, file );

			if ( TransferFileItem != 0 )
			{
				UserFileInfo->eFileState = TransferFileItem->eState;
				res = TRUE;
			}
		}
		else
		{
			res = TRUE;
		}
	}

	UnLock();

	return res;
}

/** */
void CDownloadManager::TryConnect( CString nick, CString hubname, CString hubhost )
{
	DCTransferItem * TransferItem;

	Lock();

	TransferItem = GetTransferItem(nick,hubname,hubhost);

	if ( TransferItem != 0 )
	{
		if ( TransferItem->eState == etwsIDLE )
		{
			TransferItem->timeTimeout = 0;
		}
	}

	UnLock();
}

/** */
bool CDownloadManager::RemoveTransferQueue( CString nick, CString hubname, CString hubhost, CString file )
{
	bool res = FALSE;
	DCTransferItem * TransferItem;
	DCTransferFileItem * TransferFileItem;

	Lock();

	TransferItem = GetTransferItem(nick,hubname,hubhost);

	if ( TransferItem != 0 )
	{
		if ( (TransferItem->eState == etwsIDLE) && (file == "") )
		{
			// remove complete ...
			TransferFileItem = 0;
			while( (TransferFileItem=TransferItem->pFileList->Next(0)) != 0 )
			{
				if ( TransferFileItem->eState == etfsTRANSFER )
				{
					printf("warning ! file transfer is running ...\n");//fflush(stdout);
					break;
				}

				SendFileInfo( TransferItem, TransferFileItem, TRUE );
				TransferItem->pFileList->Del(TransferFileItem);
			}

			if ( TransferItem->pTransferList->Count() == 0 )
			{
				pTransferItemList->Del(TransferItem);
				res = TRUE;
			}
			else
			{
				printf("warning ! found transfers ...\n");//fflush(stdout);
			}
		}
		else if ( file != "" )
		{
			TransferFileItem = GetTransferFileItem( TransferItem, file );

			if ( TransferFileItem != 0 )
			{
				if ( TransferFileItem->eState != etfsTRANSFER )
				{
					SendFileInfo( TransferItem, TransferFileItem, TRUE );
					TransferItem->pFileList->Del(TransferFileItem);
					res = TRUE;
				}
			}
		}
	}

	UnLock();

	return res;
}

/** */
void CDownloadManager::AddTransfer( CString host, int port, CString hubname, CString hubhost )
{
	Lock();

	CTransfer * Transfer = new CTransfer();

	Transfer->SetNick( pDCLibConfig->GetNick() );

	Transfer->SetHubName(hubname);
	Transfer->SetHubHost(hubhost);

	Transfer->SetHost( host, port );
	Transfer->SetRate( pDCLibConfig->GetMaxTransferrate() );

	pTransferList->Add(Transfer);
	Transfer->SetCallBackFunction(DC_TransferCallBack);

	Transfer->Connect();

	UnLock();
}

/** */
ulonglong CDownloadManager::GetNewTransferID()
{
	lTransferID++;

	if (lTransferID==0)
		lTransferID++;

	return lTransferID;
}

/** */
DCTransferItem * CDownloadManager::GetTransferItem( CString nick, CString hubname, CString hubhost )
{
	DCTransferItem * TransferItem = 0;

	while( (TransferItem=pTransferItemList->Next(TransferItem)) != 0 )
	{
		if ( (TransferItem->sNick    == nick) &&
		     (TransferItem->sHubName == hubname) &&
		     (TransferItem->sHubHost == hubhost) )
		{
			break;
		}
	}

	return TransferItem;
}

/** */
DCTransferFileItem * CDownloadManager::GetTransferFileItem( DCTransferItem * TransferItem, CString remotename )
{
	DCTransferFileItem * TransferFileItem = 0;

	while( (TransferFileItem=TransferItem->pFileList->Next(TransferFileItem)) != 0 )
	{
		if ( TransferFileItem->sRemoteName == remotename )
		{
			break;
		}
	}

	return TransferFileItem;
}

/** */
void CDownloadManager::AddWaitTransfer( CString nick, CString hubname, CString hubhost,
				CString remotename, CString localname,
				CString localpath, CString localrootpath,
				eltMedium medium, ulonglong size,
				ulonglong startposition, ulonglong endposition,
				bool )
{
	DCTransferItem * TransferItem;
	DCTransferFileItem * TransferFileItem = 0;

	printf("add wait transfer: %s %s %s %s %d %s %s %s %Ld %Ld %Ld\n",
		nick.Data(),
		remotename.Data(),
		hubname.Data(),
		hubhost.Data(),
		medium,
		localname.Data(),
		localpath.Data(),
		localrootpath.Data(),
		startposition,
		endposition,
		size );

	Lock();

	TransferItem = GetTransferItem( nick, hubname, hubhost );

	// no transfer found for the nick/hub
	if ( TransferItem == 0 )
	{
		TransferItem = new DCTransferItem();

		TransferItem->sNick       = nick;
		TransferItem->sHubHost    = hubhost;
		TransferItem->sHubName    = hubname;
		TransferItem->eState      = etwsIDLE;
		TransferItem->timeTimeout = 0;
		TransferItem->ID          = GetNewTransferID();

		pTransferItemList->Add(TransferItem);

		printf("transfer add\n");//fflush(stdout);
	}
	else
	{
		TransferFileItem = GetTransferFileItem( TransferItem, remotename );
	}

	// file not found for the nick/hub
	if ( TransferFileItem == 0 )
	{
		TransferFileItem = new DCTransferFileItem();

		TransferFileItem->lSize          = size;
		TransferFileItem->lStartPosition = startposition;
		TransferFileItem->lCurrentPosition = startposition;
		TransferFileItem->lEndPosition   = endposition;
		TransferFileItem->eMedium        = medium;
		TransferFileItem->sRemoteName    = remotename;
		TransferFileItem->sLocalPath     = localpath;
		TransferFileItem->sLocalName     = localname;
		TransferFileItem->sLocalRootPath = localrootpath;
		TransferFileItem->eState         = etfsNONE;
		TransferFileItem->ID             = GetNewTransferID();

		TransferItem->pFileList->Add(TransferFileItem);	

		printf("file add %ld\n",TransferItem->pFileList->Count());//fflush(stdout);
	}

	SendFileInfo( TransferItem, TransferFileItem );

	UnLock();
}

/** */
bool CDownloadManager::CheckWaitTransfer( CTransfer * Transfer )
{
	bool res = FALSE;
	DCTransferItem * TransferItem = 0;

	printf("CheckWaitTransfer: %s on %s %s\n",
		Transfer->GetDstNick().Data(),
		Transfer->GetHubName().Data(),
		Transfer->GetHubHost().Data() );

	Lock();

	// search transfer for the user in the transferlist
	while( (TransferItem=pTransferItemList->Next(TransferItem)) != 0 )
	{
		printf("Waiting: %s on %s %s\n",
			TransferItem->sNick.Data(),
			TransferItem->sHubName.Data(),
			TransferItem->sHubHost.Data() );

		// check nick
		if ( TransferItem->sNick == Transfer->GetDstNick() )
		{
			if ( TransferItem->eState != etwsWAIT )
			{
				continue;
			}

			// check hub, hub is "" on active/incoming connection
			if ( "" != Transfer->GetHubName() )
			{
				if ( (TransferItem->sHubName != Transfer->GetHubName()) ||
		     		     (TransferItem->sHubHost != Transfer->GetHubHost()) )
				{
					continue;
				}
			}
			else
			{
				// set from transfer wait item
				Transfer->SetHubHost(TransferItem->sHubHost);
				Transfer->SetHubName(TransferItem->sHubName);
			}

			// waiting transfer found ...
			res = TRUE;

			TransferItem->pTransferList->Add(Transfer);
			pTransferList->Remove(Transfer);
			TransferItem->eState = etwsRUN;

			printf("%ld waiting transfers ...\n",TransferItem->pFileList->Count());

			SendFileInfo( TransferItem );

			break;
		}
	}

	UnLock();

	return res;
}

/** */
bool CDownloadManager::SetNextFile( CTransfer * Transfer )
{
	bool res;

	Lock();
	res = SetFile(Transfer);
	UnLock();

	return res;
}

/** */
bool CDownloadManager::SetDirection( CTransfer * Transfer )
{
	int i;
	bool res = FALSE;

	// check incoming transfer limit
	i = pDCLibConfig->GetMaxUpload();

	if ( i != 0 )
	{
		if ( Transfer->GetSrcDirection() == edUPLOAD )
		{
			if ( i > lUploadCount )
			{
				res = TRUE;
			}
			else
			{
				res = CheckUserSlot( Transfer->GetDstNick(), Transfer->GetHubName() );
			}

			if (res)
			{
				lUploadCount++;
			}
		}
		else
		{
			res = TRUE;
		}
	}
	else
	{
		res = TRUE;
	}

	return res;
}

/** */
bool CDownloadManager::SetFile( CTransfer * Transfer )
{
	bool res;
	DCTransferItem * TransferItem;

	res = FALSE;

	TransferItem = GetTransferItem( Transfer->GetDstNick(), Transfer->GetHubName(), Transfer->GetHubHost() ) ;

	if ( TransferItem != 0 )
	{
		if ( TransferItem->pFileList->Count() > 0 )
		{
			DCTransferFileItem * TransferFileItem = 0;

			while( (TransferFileItem=TransferItem->pFileList->Next(TransferFileItem)) != 0 )
			{
				if ( TransferFileItem->eState == etfsNONE )
				{
					printf("set file: '%s'\n",TransferFileItem->sRemoteName.Data());//fflush(stdout);

					CString sPath="",sLocalPath="",sFile="";
					CDir dir;

					if ( TransferFileItem->eMedium == eltFILE )
					{
						// set local download root path
						if ( TransferFileItem->sLocalRootPath != "" )
						{
							sLocalPath = TransferFileItem->sLocalRootPath.Data();
						}
						else
						{
							sLocalPath = pDCLibConfig->GetDownloadFolder().Data();
						}

						// add special download path
						if ( TransferFileItem->sLocalPath != "" )
						{
							sPath += CString("/") + TransferFileItem->sLocalPath.Data();
						}

						sPath = dir.SimplePath(sPath);

						// create the path
						dir.SetPath(sLocalPath);
						if ( dir.CreatePath(sPath) == FALSE )
						{
							pDownloadManager->SendLogInfo( "Create path failed: " + sPath, Transfer );
						}
						else
						{
							sFile = sLocalPath + "/" + sPath + "/" + TransferFileItem->sLocalName.Data();
							sFile = dir.SimplePath(sFile);

							if ( TransferFileItem->lCurrentPosition > dir.getFileSize(sFile,FALSE) )
							{
								TransferFileItem->eState = etfsERROR;
								SendFileInfo( TransferItem, TransferFileItem );
								pDownloadManager->SendLogInfo( "Wrong filesize: " + sFile, Transfer );
							}
							else
							{
								printf("DOWNLOAD: '%s' %Ld %Ld %Ld %Ld '%s'\n",
								TransferFileItem->sRemoteName.Data(),
								TransferFileItem->lCurrentPosition,
								TransferFileItem->lStartPosition,
								TransferFileItem->lEndPosition,
								TransferFileItem->lSize,
								sFile.Data());//fflush(stdout);

								res = TRUE;
							}
						}
					}
					else
					{
						res = TRUE;
					}

					if ( res == TRUE )
					{
						// mark file as transfered
						TransferFileItem->eState = etfsTRANSFER;

						Transfer->SetMedium(TransferFileItem->eMedium);

						Transfer->StartDownload(TransferFileItem->sRemoteName,TransferFileItem->lStartPosition,
									TransferFileItem->lCurrentPosition,TransferFileItem->lEndPosition,TransferFileItem->lSize,sFile);
					}
					else
					{
						continue;
					}

					SendFileInfo( TransferItem, TransferFileItem );

					Transfer->SetDone(0);

					break;
				}
				else
				{
//					printf("state file: '%s' %d\n",TransferFileItem->sRemoteName.Data(),TransferFileItem->eState);//fflush(stdout);
				}
			}
		}
	}

	if ( res == FALSE )
	{
		if ( Transfer->GetDone() != 2 )
		{
			Transfer->SetDstFilename("");
			Transfer->SetSrcFilename("");
			Transfer->SetStartTime(time(0));
			Transfer->SetDone(2);
		}
	}

	return res;
}

/** */
void CDownloadManager::UpdateFileState( CTransfer * Transfer, eTransferFileState eState )
{
	DCTransferItem * TransferItem;
	DCTransferFileItem * TransferFileItem;

	Lock();

	TransferItem = GetTransferItem( Transfer->GetDstNick(), Transfer->GetHubName(), Transfer->GetHubHost() );

	if ( TransferItem != 0 )
	{
		TransferFileItem = GetTransferFileItem( TransferItem, Transfer->GetDstFilename() );

		if ( TransferFileItem != 0 )
		{
			if ( TransferFileItem->eState != etfsTRANSFER )
			{
				printf("warning, wrong state in updatefile\n");
			}

			TransferFileItem->eState = eState;
			TransferFileItem->lStartPosition = Transfer->GetStartPosition();
			TransferFileItem->lCurrentPosition = Transfer->GetCurrentPosition();
			TransferFileItem->lEndPosition = Transfer->GetEndPosition();
			TransferFileItem->lSize = Transfer->GetLength();

			printf("%Ld %Ld %Ld %Ld\n",TransferFileItem->lStartPosition,TransferFileItem->lCurrentPosition,TransferFileItem->lEndPosition,TransferFileItem->lSize);////fflush(stdout);

			if ( (TransferFileItem->lCurrentPosition == TransferFileItem->lEndPosition) && (TransferFileItem->lSize != 0) )
			{
				SendLogInfo( "Transfer done '" + TransferFileItem->sRemoteName + "'", Transfer );
				SendFileInfo( TransferItem, TransferFileItem, TRUE );
				TransferItem->pFileList->Del(TransferFileItem);
			}
			else
			{
				SendFileInfo( TransferItem, TransferFileItem );
			}
		}
	}

	UnLock();
}

/** */
void CDownloadManager::FileListDone( CTransfer * Transfer )
{
	DCTransferItem * TransferItem;
	CByteArray ba;

	Lock();

	if ( Transfer->GetBuffer(&ba) != 0 )
	{
		TransferItem = GetTransferItem( Transfer->GetDstNick(), Transfer->GetHubName(), Transfer->GetHubHost() );

		if ( TransferItem != 0 )
		{
			TransferItem->sUserFileList.Set((char*)ba.Data(),ba.Size());

			CMessageDMFileListObject * flo = new CMessageDMFileListObject();

			flo->sNick         = TransferItem->sNick;
			flo->sHubName      = TransferItem->sHubName;
			flo->sHubHost      = TransferItem->sHubHost;
			flo->sUserFileList = TransferItem->sUserFileList;

			flo->eType = DC_MESSAGE_FILELIST_OBJECT;

			if ( DC_DownloadManagerCallBack(flo) == -1 )
				delete flo;
		}
	}

	UnLock();
}

/** */
void CDownloadManager::Thread( CObject * )
{
	bool bShutdown = FALSE;
	DCTransferItem * TransferItem = 0, * tTransferItem = 0;
	CTransfer * Transfer;

	if ( iShutdownState == 2 )
	{
#ifdef WIN32
		Sleep(100);
#else
		usleep(100);
#endif
		return;
	}

	Lock();

	if ( iShutdownState == 1 )
	{
		if ( pTransferList->Count() == 0 )
		{
			bShutdown = TRUE;
		}
	}

	Transfer = 0;

	while( (Transfer=pTransferList->Next(Transfer)) != 0 )
	{
		// remove transfers
		if ( Transfer->GetDone() == 1 )
		{
			if ( Transfer->GetSrcDirection() == edUPLOAD )
			{
				lUploadCount--;
			}

			SendTransferInfo( Transfer, TRUE );
			pTransferList->Del(Transfer);
		}
	}

	while( (TransferItem=pTransferItemList->Next(TransferItem)) != 0 )
	{
		Transfer = 0;

		if ( iShutdownState == 1 )
		{
			if ( TransferItem->pTransferList->Count() != 0 )
			{
				bShutdown = FALSE;
			}
		}

		// check all transfers from the user entry
		while ( (Transfer=TransferItem->pTransferList->Next(Transfer)) != 0 )
		{
			// done==2 set new file
			if ( (Transfer->GetDone() == 2) && (iShutdownState == 0) )
			{
				if ( SetFile(Transfer) == FALSE )
				{
					// no new file ...
					if ( Transfer->GetStartTime() != 0 )
					{
						if ( (time(0)-Transfer->GetStartTime()) >= 60 )
						{
							Transfer->SetStartTime(0);
							Transfer->Disconnect(TRUE);
						}
					}
				}
			}
			else if ( Transfer->GetDone() == 1 )
			{
				if ( Transfer->GetSrcDirection() == edUPLOAD )
				{
					lUploadCount--;
				}

				SendTransferInfo( Transfer, TRUE );
				TransferItem->pTransferList->Del(Transfer);

				// set to idle (this work only with 1 transfer ...)
				TransferItem->eState = etwsIDLE;
				TransferItem->timeTimeout = time(0);
				SendFileInfo( TransferItem );
			}
		}

		// check for empty filelists and remove
		if ( (TransferItem->pFileList->Count() == 0) && (TransferItem->pTransferList->Count() == 0) )
		{
			// remove transfer item
			pTransferItemList->Del(TransferItem);
			TransferItem = tTransferItem;
		}
		else if ( (TransferItem->eState == etwsIDLE) && (iShutdownState == 0) )
		{
			if ( TransferItem->timeTimeout == 0 )
			{
				// send connection request to server ...
				if ( pDCLibServerManager->SendConnectionRequest( TransferItem->sNick, TransferItem->sHubName ) == TRUE )
				{
					TransferItem->eState = etwsWAIT;
					SendFileInfo( TransferItem );
				}
				else
				{
					printf("send connection request failed ...\n");//fflush(stdout);
				}

				if ( TransferItem->timeTimeout != time(0) )
				{
					TransferItem->timeTimeout = time(0);
					SendFileInfo( TransferItem );
				}
			}
			else if ( (time(0)-TransferItem->timeTimeout) >= pDCLibConfig->GetTransferResponseTimeout() )
			{
				TransferItem->timeTimeout = 0;
				SendFileInfo( TransferItem );
			}
			else
			{
			}
		}
		else if ( (TransferItem->eState == etwsWAIT) && (iShutdownState == 0) )
		{
			// after X seconds resend connecttome
			if ( (time(0)-TransferItem->timeTimeout) >= pDCLibConfig->GetTransferResendTimeout() )
			{
				// set to idle
				TransferItem->eState = etwsIDLE;
				TransferItem->timeTimeout = 0;
				SendFileInfo( TransferItem );
			}
		}

		tTransferItem = TransferItem;
	}

	UnLock();

	if ( iShutdownState == 1 )
	{
		if ( bShutdown == TRUE )
			iShutdownState = 2;
		else
#ifdef WIN32
			Sleep(100);
#else
			usleep(100);
#endif
	}
	else
	{
#ifdef WIN32
		Sleep(100);
#else
		usleep(1000*100);
#endif
	}
}

/** callback function */
int CDownloadManager::DMListenCallBack( int handle )
{
	if ( handle == -1 )
	{
		return -1;
	}

	pDownloadManager->SendLogInfo("Incoming connection ...\n");

	CTransfer * Transfer = new CTransfer();

	Transfer->SetNick( pDCLibConfig->GetNick() );
	Transfer->SetRate( pDCLibConfig->GetMaxTransferrate() );
	Transfer->SetCallBackFunction(DC_TransferCallBack);

	pDownloadManager->Lock();

	if ( Transfer->SetSocket(handle) == 0 )
	{
		pDownloadManager->pTransferList->Add(Transfer);
	}
	else
	{
		delete Transfer;
		close(handle);
	}

	pDownloadManager->UnLock();

	return 0;
}

/** */
int CDownloadManager::DC_TransferCallBack( CTransfer * Transfer, CObject * Object )
{
	pDownloadManager->TransferCallBackThread.Lock();

	CDCMessage *DCMsg;

	DCMsg = (CDCMessage*)Object;

	switch ( DCMsg->eType )
	{
		case DC_MESSAGE_MYNICK:
		{
			// check for download or upload
			if ( pDownloadManager->CheckWaitTransfer( Transfer ) == FALSE )
			{
				Transfer->SetSrcDirection(edUPLOAD);

				if ( pDownloadManager->SetDirection(Transfer) == FALSE )
				{
					pDownloadManager->SendLogInfo( "Warning no more free slots", Transfer );
					Transfer->SendMaxedOut();
					Transfer->SetSrcDirection(edNONE);
				}
			}
			else
			{
				Transfer->SetSrcDirection(edDOWNLOAD);
			}

			if ( Transfer->GetSrcDirection() != edNONE )
			{
				Transfer->SendDirection( Transfer->GetSrcDirection(), Transfer->GetLevel() );
			}

			break;
		}

		case DC_MESSAGE_GET:
		{
			CMessageGet * msg = (CMessageGet*)Object;

			if ( Transfer->GetSrcDirection() == edUPLOAD )
			{
				pDownloadManager->SendLogInfo( "Upload: " + msg->sFilename, Transfer );

				if ( msg->sFilename == CString(DC_USER_FILELIST) )
				{
					if ( pDCLibConfig->GetShareSize() == 0 )
					{
						Transfer->SendError("File not found.");
					}
					else
					{
						CByteArray ba;

						if ( pDCLibConfig->GetShareBuffer( &ba ) == 0 )
						{
							// set send buffer
							Transfer->SetBuffer(&ba);
							// set local transfer medium to buffer
							Transfer->SetMedium(eltBUFFER);
							// start upload
							Transfer->StartUpload( msg->sFilename, pDCLibConfig->GetShareBufferSize(), msg->lPos-1, "" );
						}
					}
				}
				else
				{
					CString s;
					ulonglong  len;
					CDir dir;
					CList<CString> SharedFolders;

					pDCLibConfig->GetSharedFolders(&SharedFolders);

					// search file and upload ...
					if ( (s = dir.GetCorrectFilePath(msg->sFilename.Data(),&SharedFolders)) == "" )
					{
						Transfer->SendError("File not found.");

						pDownloadManager->SendLogInfo( "Upload (File not found): " + msg->sFilename, Transfer );
					}
					else
					{
						len = dir.getFileSize(s,FALSE);

						if ( msg->lPos >= len )
						{
							Transfer->SendError("File allready download.");
						}
						else if ( msg->lPos == 0 )
						{
							Transfer->SendError("Wrong file position.");
						}
						else
						{
							// set local transfer medium to buffer
							Transfer->SetMedium(eltFILE);
							// start upload
							Transfer->StartUpload( msg->sFilename, len, msg->lPos-1, s );
						}
					}
				}
			}
			else
			{
				pDownloadManager->SendLogInfo( "Warning wrong mode", Transfer );
				Transfer->Disconnect(TRUE);
			}

			break;
		}

		case DC_MESSAGE_DIRECTION:
		{
			CMessageDirection * msg = (CMessageDirection*)Object;

			printf("DIRECTION: file : %s\n",Transfer->GetDstFilename().Data());
			printf("DIRECTION: level: %d %d\n",Transfer->GetLevel(), msg->iLevel);
			printf("DIRECTION: direc: %d %d\n",Transfer->GetSrcDirection(), msg->Direction);

			if ( (Transfer->GetLevel() < msg->iLevel) &&
				 (msg->Direction == Transfer->GetSrcDirection()) )
			{
				if ( Transfer->GetSrcDirection() == edUPLOAD )
				{
					pDownloadManager->lUploadCount--;
					Transfer->SetSrcDirection(edDOWNLOAD);
				}
				else if ( Transfer->GetSrcDirection() == edDOWNLOAD )
				{
					if ( pDownloadManager->SetDirection(Transfer) == FALSE )
					{
						pDownloadManager->SendLogInfo( "Warning no more free slots", Transfer );
						Transfer->SendMaxedOut();
						Transfer->SetSrcDirection(edNONE);
					}
					else
					{
						Transfer->SetSrcDirection(edUPLOAD);
					}
				}

				printf("DIRECTION: dirne: %d %d\n",Transfer->GetSrcDirection(), msg->Direction);
			}
			else
			{
			}

			if ( Transfer->GetSrcDirection() == edDOWNLOAD )
			{
				printf("DIRECTION: download mode ...\n");

				// next file
				if ( pDownloadManager->SetNextFile(Transfer) == FALSE )
				{
					printf("DIRECTION: download mode without destination file -> disconnecting!\n");
					Transfer->Disconnect(TRUE);
				}
			}
			else
			{
				printf("DIRECTION: we are in upload mode ...\n");
			}

			//fflush(stdout);
			break;
		}

		case DC_MESSAGE_CONNECTION_STATE:
		{
			bool remove = TRUE;
			CMessageConnectionState *msg = (CMessageConnectionState*)Object;

			if ( msg->eState == estCONNECTED )
			{
				remove = FALSE;
			}
			else if ( msg->eState == estDISCONNECTED )
			{
			}
			else if ( msg->eState == estSOCKETERROR )
			{
				remove = FALSE;
			}
			else if ( msg->eState == estCONNECTIONTIMEOUT )
			{
				remove = FALSE;
			}

			if ( (remove == TRUE) && (Transfer->GetSrcDirection() == edDOWNLOAD) )
			{
				if ( Transfer->GetMedium() == eltBUFFER )
					Transfer->SetStartPosition(0);
				pDownloadManager->UpdateFileState(Transfer,etfsNONE);
			}

			if ( remove == TRUE )
			{
				Transfer->SetDone(1);
			}

			break;
		}

		case DC_MESSAGE_TRANSFER:
		{
			CMessageTransfer *msg = (CMessageTransfer*)Object;

//			ulonglong rate;
			CString s;
			CString r;

			pDownloadManager->SendTransferInfo(Transfer);

			// filetransfer done
			if ( msg->lTransfered == msg->lLength )
			{
				// download ready
				if ( Transfer->GetSrcDirection() == edDOWNLOAD )
				{
					pDownloadManager->UpdateFileState(Transfer,etfsNONE);

					// show user file list
					if ( Transfer->GetDstFilename() == CString(DC_USER_FILELIST) )
					{
						pDownloadManager->FileListDone(Transfer);

						if ( pDownloadManager->SetNextFile(Transfer) == FALSE )
						{
						}
					}
					else
					{
						// set next file for download
						if ( pDownloadManager->SetNextFile(Transfer) == FALSE )
						{
						}
					}
				}
			}

			break;
		}

		case DC_MESSAGE_FILELENGTH:
		{
//			CMessageFileLength * msg = (CMessageFileLength*)Object;

			pDownloadManager->SendTransferInfo(Transfer);

			break;
		}

		case DC_MESSAGE_MAXEDOUT:
		{
			pDownloadManager->SendLogInfo( "Busy", Transfer );
			Transfer->Disconnect(TRUE);
			break;
		}

		case DC_MESSAGE_GETLISTLEN:
		{
			Transfer->SendListLen(pDCLibConfig->GetShareBufferSize());
			break;
		}

		case DC_MESSAGE_ERROR:
		{
			CMessageError * msg = (CMessageError*)Object;

			pDownloadManager->SendLogInfo( "Error: " + msg->sError, Transfer );

			if ( Transfer->GetSrcDirection() == edDOWNLOAD )
			{
				pDownloadManager->UpdateFileState(Transfer,etfsERROR);

				// set next file for download
				if ( pDownloadManager->SetNextFile(Transfer) == FALSE )
				{
				}
			}
			break;
		}

		default:
		{
			printf("dctransfer unknown message: %d\n",DCMsg->eType);
			break;
		}
	}

	delete Object;

	pDownloadManager->TransferCallBackThread.UnLock();

	return 0;
}
