/***************************************************************************
                          cdownloadmanager.h  -  description
                             -------------------
    begin                : Don Mai 16 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CDOWNLOADMANAGER_H
#define CDOWNLOADMANAGER_H

#include <dclib/clist.h>
#include <dclib/ctransfer.h>
#include <dclib/cthread.h>
#include <dclib/clisten.h>
#include <dclib/cdownloadobjects.h>

/**
  *@author Mathias Kster
  */

class  DLL_EXPORT CUserFileInfo : public CObject {
public:
	/** */
	CUserFileInfo() {};
	/** */
	virtual ~CUserFileInfo() {};
	/** */
	eTransferWaitState eWaitState;
	/** */
	eTransferFileState eFileState;
	/** */
	CString sUserFileList;
};

class  DLL_EXPORT CExtraUserSlot : public CObject {
public:
	/** */
	CExtraUserSlot() {};
	/** */
	virtual ~CExtraUserSlot() {};
	/** */
	CString sNick;
	/** */
	CString sHubName;
	/** */
	int iSlots;
	/** */
	bool bPermanent;
};

#ifdef WIN32
template class DLL_EXPORT CList<CExtraUserSlot>;
#endif

class  DLL_EXPORT CDownloadManager : public CThread {
public: 
	CDownloadManager();
	virtual ~CDownloadManager();

	/** */
	int LoadQueue();
	/** */
	int SaveQueue();
	/** disconnect all transfers for a clean shutdown */
	void Shutdown();
	/** */
	int GetShutdownState() const;
	/** */
	void AddWaitTransfer( CString nick, CString hubname, CString hubhost,
				CString remotename, CString localname,
				CString localpath, CString localrootpath,
				eltMedium medium, ulonglong size,
				ulonglong startposition, ulonglong endposition,
				bool multi = FALSE );
	/** */
	bool GetUserFileInfo( CString nick, CString hubname, CString hubhost, CString file, CUserFileInfo * UserFileInfo );
	/** */
	void TryConnect( CString nick, CString hubname, CString hubhost );
	/** */
	bool RemoveTransferQueue( CString nick, CString hubname, CString hubhost, CString file );
	/** */
	void AddTransfer( CString host, int port, CString hubname, CString hubhost );
	/** */
	bool CloseTransfer( CString nick, CString host, CString file );
	/** */
	bool InitListen( CString & s );
	/** */
	void StopListen();
	/** */
	void AddUserSlot( CString nick, CString hubname, int slot, bool permanent = FALSE );

	/** thread callbackfunction */
	virtual void Thread( CObject * Object );
	/** listen callback function */
	static int DMListenCallBack( int handle );
	/** callback function */
	virtual int DC_DownloadManagerCallBack( CObject * ) { return -1; };

protected:
	/** */
	static int DC_TransferCallBack( CTransfer * Transfer, CObject * Object );

private:
	/** */
	DCTransferItem * GetTransferItem( CString nick, CString hubname, CString hubhost );
	/** */
	DCTransferFileItem * GetTransferFileItem( DCTransferItem * TransferItem, CString remotename );

	/** */
	void SendFileInfo( DCTransferItem * TransferItem, DCTransferFileItem * TransferFileItem = 0, bool bRemoveFile = FALSE );
	/** */
	void SendTransferInfo( CTransfer * Transfer, bool remove = FALSE );
	/** */
	void SendSlotInfo( CExtraUserSlot * Object );
	/** */
	void SendLogInfo( CString message, CTransfer * Transfer = 0 );
	/** */
	bool CheckUserSlot( CString nick, CString hubname );

	/** */
	bool CheckWaitTransfer( CTransfer * Transfer );
	/** */
	bool SetNextFile( CTransfer * Transfer );
	/** */
	bool SetDirection( CTransfer * Transfer );
	/** */
	bool SetFile( CTransfer * Transfer );
	/** */
	void UpdateFileState( CTransfer * Transfer, eTransferFileState eState );
	/** */
	void FileListDone( CTransfer * Transfer );

	/** */
	ulonglong GetNewTransferID();

	/** */
	int iShutdownState;
	/** */
	CThread TransferCallBackThread;
	/** */
	CThread LogThread;
	/** */
	ulonglong lTransferID;
	/** */
	long lUploadCount;
	/** */
	CList<DCTransferItem> *pTransferItemList;
	/** */
	CList<CTransfer> *pTransferList;
	/** */
	CListen Listen;
	/** */
	CList<CExtraUserSlot> *pExtraUserSlotList;
};

/** */
inline int CDownloadManager::GetShutdownState() const
{ return iShutdownState; }

#endif
